//Accel library
#include <AccelStepper.h>

//Stepper definieren (oder deklarieren?)
AccelStepper stepper1(AccelStepper::FULL2WIRE, 4, 5); // positiv -  vorwärts (Rechts)
AccelStepper stepper2(AccelStepper::FULL2WIRE, 2, 3); // positiv - rückwärts (Links)
//Hinweise:
// Pin 2 und Pin 4 sind die Step-pins
// Pin 3 und Pin 5 die Richtungs-Pins

//Ultraschall Pins
//rechts
const int trig1Pin = 11;
const int echo1Pin = 12;
//links
const int trig2Pin = 18;
const int echo2Pin = 13;

void setup() {

  //USM Pins
  //Rechts:
  pinMode(trig1Pin, OUTPUT);
  pinMode(echo1Pin, INPUT);
  //Links:
  pinMode(trig2Pin, OUTPUT);
  pinMode(echo2Pin, INPUT);

  //Stepper
  //rechts
  stepper1.setMaxSpeed(2000.0);
  stepper1.setSpeed(1000.0);// Schritte pro Minute


  //links
  stepper2.setMaxSpeed(2000.0);
  stepper2.setSpeed(1000.0); // Schritte pro Minute

  //Hinweise:
  //setMaxSpeed sollte bei allen Steppern der Einfachheit halber immer das gleiche Vorzeichen an der Zahl haben
  //bei setSpeed kann man im setup schon variieren, allerdings sind gleiche Vorzeichen auch hier zu empfehlen, wenn man später im loop den Motor in verschiedene Richtungen drehen lassen möchte
  //Die Richtung in die sich die Räder an den Motoren drehen lässt sich durch eine Vorzeichenänderung bei setSpeed und/oder setMaxSpeed umkehren

  //Stepper, Blink
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);

  Serial.begin(115200);
}

void loop() {

  double EntfLinks = USMLinks(); // Entfernung Links
  double EntfRechts = USMRechts();  // Entfernung Rechts

  // Geradeausfahren
  while ( (EntfLinks > 10 || EntfLinks == 0) && (EntfRechts > 10  || EntfRechts == 0) ) {
    for (int i = 0; i < 50; i++) {
      stepper1.setSpeed(-300.0);
      stepper2.setSpeed(300.0);
      stepper1.runSpeed();
      stepper2.runSpeed();
    }
    EntfLinks = USMLinks();
    EntfRechts = USMRechts();

  }

  //Ultraschall
  while ((USMLinks() <= 10) && (USMLinks() != 0)) {
    stepper1.setSpeed(-300.0);
    stepper2.setSpeed(350.0);
    for (int i = 0; i < 15; i++) {
      stepper1.runSpeed();
      stepper2.runSpeed();
    }
  }
  while ((USMRechts() <= 10) && (USMRechts() != 0)) {
    stepper1.setSpeed(-350.0);
    stepper2.setSpeed(300.0);
    for (int i = 0; i < 15; i++) {
      stepper1.runSpeed();
      stepper2.runSpeed();
    }
  }

}

//Ultraschallsensoren (Abstandsmesser)

double USMRechts() {

  long zeit, entf;

  digitalWrite(trig1Pin, LOW);
  delayMicroseconds(2);
  digitalWrite(trig1Pin, HIGH);// Trigger sendet 10 Microsekunden ein Signal aus
  delayMicroseconds(10);
  digitalWrite(trig1Pin, LOW);
  zeit = pulseIn(echo1Pin, HIGH);

  entf = zeit / 29.1 / 2 ;

  if (entf <= 0) {
    Serial.println("Out of range");
  }
  else {
    Serial.print(entf);
    Serial.print("cm");
    Serial.println();
  }

  return entf;
}

double USMLinks() {

  long zeit, entf;

  digitalWrite(trig2Pin, LOW);
  delayMicroseconds(2);
  digitalWrite(trig2Pin, HIGH);// Trigger sendet 10 Microsekunden ein Signal aus
  delayMicroseconds(10);
  digitalWrite(trig2Pin, LOW);
  zeit = pulseIn(echo2Pin, HIGH); // Zeitbeschränkung von 600 mikrosekunden, Messung also bis c.a. 10 cm

  entf = zeit / 29.1 / 2 ;

  if (entf <= 0) {
    Serial.println("Out of range");
  }
  else {
    Serial.print(entf);
    Serial.print("cm");
    Serial.println();
  }

  return entf;
}


// Blink Ansteuerung der Motoren
//Hinweise:
//2 und 4 sind Step Pins
//3 und 5 sind Richtungs pins
void SchrittRechtsVorwarts() {
  digitalWrite(4, HIGH);
  digitalWrite(5, HIGH);
  delay(5);
  digitalWrite(4, LOW);
  digitalWrite(5, LOW);
  delay(10);
}
void SchrittRechtsRuckwarts() {
  digitalWrite(4, HIGH);
  digitalWrite(5, LOW);
  delay(5);
  digitalWrite(5, HIGH);
  digitalWrite(4, LOW);
  delay(10);
}

void SchrittLinksVorwarts() {
  digitalWrite(2, HIGH);
  digitalWrite(3, LOW);
  delay(5);
  digitalWrite(2, LOW);
  digitalWrite(3, HIGH);
  delay(10);
}

void SchrittLinksRuckwarts() {
  digitalWrite(2, HIGH);
  digitalWrite(3, HIGH);
  delay(5);
  digitalWrite(2, LOW);
  digitalWrite(3, LOW);
  delay(10);
}

