//Accel library
#include <AccelStepper.h>

//Stepper definieren (oder deklarieren?)
AccelStepper stepper1(AccelStepper::FULL2WIRE, 4, 5); // positiv -  vorwärts (Rechts)
AccelStepper stepper2(AccelStepper::FULL2WIRE, 2, 3); // positiv - rückwärts (Links)
//Hinweise:
// Pin 2 und Pin 4 sind die Step-pins
// Pin 3 und Pin 5 die Richtungs-Pins

//Farbsensor Pins
#define S0 10
#define S1 9
#define S2 8
#define S3 7
#define sensorOut 6

//Ultraschall Pins
//rechts
const int trig1Pin = 11;
const int echo1Pin = 12;
//links
const int trig2Pin = 18;
const int echo2Pin = 13;

void setup() {

  // Farbsensor Pins
  pinMode(S0, OUTPUT);
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);
  pinMode(S3, OUTPUT);
  pinMode(sensorOut, INPUT);
  // Setting frequency-scaling to 20%
  digitalWrite(S0, HIGH);
  digitalWrite(S1, HIGH);


  //USM Pins
  //Rechts:
  pinMode(trig1Pin, OUTPUT);
  pinMode(echo1Pin, INPUT);
  digitalWrite(trig1Pin, LOW);
  //Links:
  pinMode(trig2Pin, OUTPUT);
  pinMode(echo2Pin, INPUT);
  digitalWrite(trig2Pin, LOW);

  //Stepper
  //rechts
  stepper1.setMaxSpeed(2000.0);
  stepper1.setSpeed(1000.0);// Schritte pro Minute


  //links
  stepper2.setMaxSpeed(2000.0);
  stepper2.setSpeed(1000.0); // Schritte pro Minute

  //Hinweise:
  //setMaxSpeed sollte bei allen Steppern der Einfachheit halber immer das gleiche Vorzeichen an der Zahl haben
  //bei setSpeed kann man im setup schon variieren, allerdings sind gleiche Vorzeichen auch hier zu empfehlen, wenn man später im loop den Motor in verschiedene Richtungen drehen lassen möchte
  //Die Richtung in die sich die Räder an den Motoren drehen lässt sich durch eine Vorzeichenänderung bei setSpeed und/oder setMaxSpeed umkehren

  //Stepper, Blink
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);

  Serial.begin(115200);
}

void loop() {

  double EntfLinks = 10; // Entfernung Links
  double EntfRechts = 10;  // Entfernung Rechts

  // Geradeausfahren
  while ( (Rot() > 10 && Blau() > 13) && (EntfLinks > 5 || EntfLinks == 0) && (EntfRechts > 5  || EntfRechts == 0) ) {
    for (int i = 0; i < 50; i++) {
      stepper1.setSpeed(-300.0);
      stepper2.setSpeed(300.0);
      stepper1.runSpeed();
      stepper2.runSpeed();
    }
    EntfLinks = USMLinks();
    EntfRechts = USMRechts();
  }

  //Ultraschall
  while ((USMLinks() <= 5) && (USMLinks() != 0)) {
    stepper1.setSpeed(-30.0);
    stepper2.setSpeed(300.0);
    for (int i = 0; i < 20; i++) {
      stepper1.runSpeed();
      stepper2.runSpeed();
    }
  }
  while ((USMRechts() <= 5) && (USMRechts() != 0)) {
    stepper1.setSpeed(-300.0);
    stepper2.setSpeed(30.0);
    for (int i = 0; i < 20; i++) {
      stepper1.runSpeed();
      stepper2.runSpeed();
    }
  }

  // RGB
  if (Rot() <= 10 ) {
    for (int i = 0; i < 150; i++) {
      SchrittLinksVorwarts();
      SchrittRechtsRuckwarts();
    }
  }
  if (Blau() <= 13) {
    for (int i = 0; i < 150; i++) {
      SchrittLinksRuckwarts();
      SchrittRechtsVorwarts();
    }
  }
}

//FUNKTIONEN

// RGB-Sensor
long Rot() {
  digitalWrite(S2, LOW);
  digitalWrite(S3, LOW);
  long FrequenzROT = pulseIn(sensorOut, LOW);
  //Serial.print("R= ");
  //Serial.print(FrequenzROT);
  //Serial.println("  ");
  return (FrequenzROT);
}

long Grun() {
  digitalWrite(S2, HIGH);
  digitalWrite(S3, HIGH);
  long FrequenzGRUN = pulseIn(sensorOut, LOW);
  //Serial.print("G= ");
  //Serial.print(FrequenzGRUN);
  //Serial.print("  ");
  //Serial.print("\t");
  return (FrequenzGRUN);
}

long Blau() {
  digitalWrite(S2, LOW);
  digitalWrite(S3, HIGH);
  long FrequenzBLAU = pulseIn(sensorOut, LOW);
  //Serial.print("B= ");
  //Serial.print(FrequenzBLAU);
  //Serial.print("  ");
  //Serial.print("\t");
  return (FrequenzBLAU);
}

//Ultraschallsensoren (Abstandsmesser)

double USMRechts() {
  double R5M = 0; // R5M = Rechts 5 Messungen  -> Variable in der die Messungen aufaddiert und gespeichert werden
  for (int x = 0; x < 5; x++) {
    double Dauer1;
    digitalWrite(trig1Pin, LOW);
    delayMicroseconds(2);
    digitalWrite(trig1Pin, HIGH);// Trigger sendet 10 Microsekunden ein Signal aus
    delayMicroseconds(10);
    digitalWrite(trig1Pin, LOW);
    Dauer1 = pulseIn(echo1Pin, HIGH);
    R5M += Dauer1; // Messungen werden aufaddiert und gespeichert
  }
  return R5M * 0.0034;// Mittelwert aus den Messungen wird errechnet, in cm umgewandelt und zurückgegeben ( R5M/5 * 0.034/2 = R5M * 0.0034)
}

double USMLinks() {
  double L5M = 0;  // L5M = Links 5 Messungen  -> Analog zu Rechts
  for (int x = 0; x < 5; x++) { // Messung wird 5 Mal durchgeführt
    double Dauer2;
    digitalWrite(trig2Pin, LOW);
    delayMicroseconds(2);
    digitalWrite(trig2Pin, HIGH);// Trigger sendet 10 Microsekunden ein Signal aus
    delayMicroseconds(10);
    digitalWrite(trig2Pin, LOW);
    Dauer2 = pulseIn(echo2Pin, HIGH); // Zeitbeschränkung von 600 mikrosekunden, Messung also bis c.a. 10 cm
    L5M += Dauer2; // Messungen werden aufaddiert und gespeichert
  }
  return L5M * 0.0034; // Mittelwert aus den Messungen wird errechnet, in cm umgewandelt und zurückgegeben( L5M/5 * 0.034/2 = L5M * 0.0034)
}


// Blink Ansteuerung der Motoren
//Hinweise:
//2 und 4 sind Step Pins
//3 und 5 sind Richtungs pins
void SchrittRechtsVorwarts() {
  digitalWrite(4, HIGH);
  digitalWrite(5, HIGH);
  delay(5);
  digitalWrite(4, LOW);
  digitalWrite(5, LOW);
  delay(10);
}
void SchrittRechtsRuckwarts() {
  digitalWrite(4, HIGH);
  digitalWrite(5, LOW);
  delay(5);
  digitalWrite(5, HIGH);
  digitalWrite(4, LOW);
  delay(10);
}

void SchrittLinksVorwarts() {
  digitalWrite(2, HIGH);
  digitalWrite(3, LOW);
  delay(5);
  digitalWrite(2, LOW);
  digitalWrite(3, HIGH);
  delay(10);
}

void SchrittLinksRuckwarts() {
  digitalWrite(2, HIGH);
  digitalWrite(3, HIGH);
  delay(5);
  digitalWrite(2, LOW);
  digitalWrite(3, LOW);
  delay(10);
}

