//Accel library
#include <AccelStepper.h>

//Stepper definieren (oder deklarieren?)
AccelStepper stepper1(AccelStepper::FULL2WIRE, 4, 5); // positiv -  vorwärts (Rechts)
AccelStepper stepper2(AccelStepper::FULL2WIRE, 2, 3); // positiv - rückwärts (Links)
//Hinweise:
// Pin 2 und Pin 4 sind die Step-pins
// Pin 3 und Pin 5 die Richtungs-Pins

//Farbsensor Pins
#define S0 10
#define S1 9
#define S2 8
#define S3 7
#define sensorOut 6

//Ultraschall Pins
//rechts
const int trig1Pin = 11;
const int echo1Pin = 12;
//links
const int trig2Pin = 18;
const int echo2Pin = 13;

void setup() {

  // Farbsensor Pins
  pinMode(S0, OUTPUT);
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);
  pinMode(S3, OUTPUT);
  pinMode(sensorOut, INPUT);
  // Setting frequency-scaling to 20%
  digitalWrite(S0, HIGH);
  digitalWrite(S1, HIGH);


  //USM Pins
  //Rechts:
  pinMode(trig1Pin, OUTPUT);
  pinMode(echo1Pin, INPUT);
  digitalWrite(trig1Pin, LOW);
  //Links:
  pinMode(trig2Pin, OUTPUT);
  pinMode(echo2Pin, INPUT);
  digitalWrite(trig2Pin, LOW);

  //Stepper
  //rechts
  stepper1.setMaxSpeed(2000.0);
  stepper1.setSpeed(500.0);// Schritte pro Minute


  //links
  stepper2.setMaxSpeed(2000.0);
  stepper2.setSpeed(500.0); // Schritte pro Minute

  //Hinweise:
  //setMaxSpeed sollte bei allen Steppern der Einfachheit halber immer das gleiche Vorzeichen an der Zahl haben
  //bei setSpeed kann man im setup schon variieren, allerdings sind gleiche Vorzeichen auch hier zu empfehlen, wenn man später im loop den Motor in verschiedene Richtungen drehen lassen möchte
  //Die Richtung in die sich die Räder an den Motoren drehen lässt sich durch eine Vorzeichenänderung bei setSpeed und/oder setMaxSpeed umkehren

  //Stepper, Blink
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);

  Serial.begin(115200);
}

void loop() {

  int rot = Rot();
  int blau = Blau();
  double EntfLinks = USMLinks(); // Entfernung Links
  double EntfRechts = USMRechts();  // Entfernung Rechts

  Serial.println(rot);
  Serial.println("\t");
  Serial.println(blau);
  Serial.println("\t");
  Serial.println(EntfLinks);
  Serial.println("\t");
  Serial.println(EntfRechts);

  // Geradeaus fahren
  if (( rot > 10 && blau > 13) && (EntfLinks > 10) && (EntfRechts > 10) ) {
    for (int i = 0; i < 25; i++) {
      stepper1.setSpeed(-300.0);
      stepper2.setSpeed(300.0);
      stepper1.runSpeed();
      stepper2.runSpeed();
    }
  }

  //Farben
  if (rot <= 10) {
    for (int i = 0; i < 140; i++) {
      SchrittRechtsRuckwarts();
      SchrittLinksVorwarts();
    }
  }
  if (blau <= 13) {
    for (int i = 0; i < 140; i++) {
      SchrittRechtsVorwarts();
      SchrittLinksRuckwarts();
    }
  }

  //Ultraschall
  if (EntfLinks <= 10) {
    stepper1.setSpeed(-300.0);
    stepper2.setSpeed(350.0);
    for (int i = 0; i < 10; i++) {
      stepper1.runS peed();
      delayMicroseconds(1000);
      stepper2.runSpeed();
      delayMicroseconds(1000);
    }
  }
  if (EntfRechts <= 10) {
    stepper1.setSpeed(-350.0);
    stepper2.setSpeed(300.0);
    for (int i = 0; i < 10; i++) {
      stepper1.runSpeed();
      delayMicroseconds(1000);
      stepper2.runSpeed();
      delayMicroseconds(1000);
    }
  }
}


//FUNKTIONEN

// RGB-Sensor
long Rot() {
  digitalWrite(S2, LOW);
  digitalWrite(S3, LOW);
  long FrequenzROT = pulseIn(sensorOut, LOW);
  //Serial.print("R= ");
  //Serial.print(FrequenzROT);
  //Serial.println("  ");
  return (FrequenzROT);
}

long Grun() {
  digitalWrite(S2, HIGH);
  digitalWrite(S3, HIGH);
  long FrequenzGRUN = pulseIn(sensorOut, LOW);
  //Serial.print("G= ");
  //Serial.print(FrequenzGRUN);
  //Serial.print("  ");
  //Serial.print("\t");
  return (FrequenzGRUN);
}

long Blau() {
  digitalWrite(S2, LOW);
  digitalWrite(S3, HIGH);
  long FrequenzBLAU = pulseIn(sensorOut, LOW);
  //Serial.print("B= ");
  //Serial.print(FrequenzBLAU);
  //Serial.print("  ");
  //Serial.print("\t");
  return (FrequenzBLAU);
}

// Blink Ansteuerung der Motoren
//Hinweise:
//2 und 4 sind Step Pins
//3 und 5 sind Richtungs pins
void SchrittRechtsVorwarts() {
  digitalWrite(4, HIGH);
  digitalWrite(5, HIGH);
  delay(5);
  digitalWrite(4, LOW);
  digitalWrite(5, LOW);
  delay(10);
}
void SchrittRechtsRuckwarts() {
  digitalWrite(4, HIGH);
  digitalWrite(5, LOW);
  delay(5);
  digitalWrite(5, HIGH);
  digitalWrite(4, LOW);
  delay(10);
}

void SchrittLinksVorwarts() {
  digitalWrite(2, HIGH);
  digitalWrite(3, LOW);
  delay(5);
  digitalWrite(2, LOW);
  digitalWrite(3, HIGH);
  delay(10);
}

void SchrittLinksRuckwarts() {
  digitalWrite(2, HIGH);
  digitalWrite(3, HIGH);
  delay(5);
  digitalWrite(2, LOW);
  digitalWrite(3, LOW);
  delay(10);
}

void Geradeaus () {
  stepper1.setSpeed(-300.0);
  stepper2.setSpeed(300.0);
  stepper1.runSpeed();
  stepper2.runSpeed();
}

double USMRechts() {
  long zeit, entf;
  digitalWrite(trig1Pin, LOW);
  delayMicroseconds(2);
  digitalWrite(trig1Pin, HIGH);// Trigger sendet 10 Microsekunden ein Signal aus
  delayMicroseconds(10);
  digitalWrite(trig1Pin, LOW);
  zeit = pulseIn(echo1Pin, HIGH);
  entf = zeit / 29.1 / 2 ;
  return entf;
}

double USMLinks() {
  long zeit, entf;
  digitalWrite(trig2Pin, LOW);
  delayMicroseconds(2);
  digitalWrite(trig2Pin, HIGH);// Trigger sendet 10 Microsekunden ein Signal aus
  delayMicroseconds(10);
  digitalWrite(trig2Pin, LOW);
  zeit = pulseIn(echo2Pin, HIGH);
  entf = zeit / 29.1 / 2 ;
  return entf;
}

