//Accel library
#include <AccelStepper.h>
#include <Stepper.h>

#define steps 557
Stepper stepper01(steps, 2, 3);
Stepper stepper02(steps, 4, 5);

//Stepper definieren (oder deklarieren?)
AccelStepper stepper1(AccelStepper::FULL2WIRE, 2, 3); // positiv - rückwärts
AccelStepper stepper2(AccelStepper::FULL2WIRE, 4, 5); // positiv -  vorwärts

//Farbsensor Pins
#define S0 10
#define S1 9
#define S2 8
#define S3 7
#define sensorOut 6

//Ultraschall Pins
//rechts
const int trig1Pin = 11;
const int echo1Pin = 12;
//links
const int trig2Pin = 18;
const int echo2Pin = 13;

void setup() {

  // Farbsensor Pins
  pinMode(S0, OUTPUT);
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);
  pinMode(S3, OUTPUT);
  pinMode(sensorOut, INPUT);
  // Setting frequency-scaling to 20%
  digitalWrite(S0, HIGH);
  digitalWrite(S1, HIGH);


  //USM Pins
  //Rechts:
  pinMode(trig1Pin, OUTPUT);
  pinMode(echo1Pin, INPUT);
  digitalWrite(trig1Pin, LOW);
  //Links:
  pinMode(trig2Pin, OUTPUT);
  pinMode(echo2Pin, INPUT);
  digitalWrite(trig2Pin, LOW);

  //Stepper
  //rechts
  stepper1.setMaxSpeed(2000.0);
  stepper1.setSpeed(1000.0);
  //links
  stepper2.setMaxSpeed(2000.0);
  stepper2.setSpeed(1000.0);

  Serial.begin(115200);
}

void loop() {
  long EntfLinks = 10; // Entfernung Links
  long EntfRechts = 10;  // Entfernung Rechts

  // Farbsensor
  while ( (Rot() > 10 && Blau() > 13) && (EntfLinks > 5 || EntfLinks == 0) && (EntfRechts > 5  || EntfRechts == 0) ) {
    stepper1.setSpeed(1000.0);
    stepper2.setSpeed(-1000.0);
    stepper1.runSpeed();
    stepper2.runSpeed();
    EntfLinks = USMLinks();
    EntfRechts = USMRechts();

  }

  //Ultraschall
  while ((USMLinks() <= 5) && (USMLinks() != 0)) {

    stepper1.setSpeed(1700.0);
    stepper2.setSpeed(0.0);
    stepper1.runSpeed();
    stepper2.runSpeed();
  }
  while ((USMRechts() <= 5) && (USMRechts() != 0)) {

    stepper2.setSpeed(-2000.0);
    stepper1.setSpeed(0.0);
    stepper1.runSpeed();
    stepper2.runSpeed();
  }

  // RGB
  if (Rot() <= 10 ) {
    stepper1.setSpeed(1000.0);
    stepper2.setSpeed(1000.0);
    stepper1.runSpeed();
    stepper2.runSpeed();
    stepper02.step (steps);
  }


  if (Blau() <= 13) {
    stepper1.setSpeed(-1000.0);
    stepper2.setSpeed(-1000.0);
    stepper1.runSpeed();
    stepper2.runSpeed();
    stepper01.step (steps);
  }
}


//FUNKTIONEN

// RGB-Sensor
long Rot() {
  digitalWrite(S2, LOW);
  digitalWrite(S3, LOW);
  long FrequenzROT = pulseIn(sensorOut, LOW);
  Serial.print("R= ");
  Serial.print(FrequenzROT);
  Serial.println("  ");
  return (FrequenzROT);
}

long Grun() {
  digitalWrite(S2, HIGH);
  digitalWrite(S3, HIGH);
  long FrequenzGRUN = pulseIn(sensorOut, LOW);
  //Serial.print("G= ");
  //Serial.print(FrequenzGRUN);
  //Serial.print("  ");
  // Serial.print("\t");
  return (FrequenzGRUN);
}

long Blau() {
  digitalWrite(S2, LOW);
  digitalWrite(S3, HIGH);
  long FrequenzBLAU = pulseIn(sensorOut, LOW);
  Serial.print("B= ");
  Serial.print(FrequenzBLAU);
  Serial.print("  ");
  Serial.print("\t");
  return (FrequenzBLAU);
}

//Ultraschallsensoren (Abstandsmesser)

long USMRechts() {
  long MWTR = 0;
  for (int x = 0; x < 5; x++) {
    long Dauer1;
    digitalWrite(trig1Pin, LOW);
    delayMicroseconds(2);
    digitalWrite(trig1Pin, HIGH);
    delayMicroseconds(10);
    digitalWrite(trig1Pin, LOW);
    Dauer1 = pulseIn(echo1Pin, HIGH, 600);
    MWTR += Dauer1;
    // Serial.print("cm1: ");
    //Serial.print(Dauer1);
    // Serial.println();
  }
  return MWTR * 0.0034;
}

long USMLinks() {
  long MWTL = 0;  // Variable in der die Messungen aufaddiert und gespeichert werden
  for (int x = 0; x < 5; x++) { // Messung wird 5 Mal durchgeführt
    long Dauer2;
    digitalWrite(trig2Pin, LOW);
    delayMicroseconds(2);
    //trig2Pin HIGH, 10 micro seconds
    digitalWrite(trig2Pin, HIGH);
    delayMicroseconds(10);
    digitalWrite(trig2Pin, LOW);
    Dauer2 = pulseIn(echo2Pin, HIGH, 600); // Zeitbeschränkung von 600 mikrosekunden, Messung also bis c.a. 10 cm
    MWTL += Dauer2; // Messungen werden aufaddiert
    //Serial.print("cm2: ");
    //Serial.print(Dauer2);// unveränderte Weg-Zeit wird ausgegeben
    //Serial.println();
  }
  return MWTL * 0.0034; // Mittelwert aus den Messungen wird errechnet und in cm umgewandelt ( MWT/5 * 0.034/2 = MWT * 0.0034)
}
